import { MultikeyMap } from 'multikey-map';
const RESOLVED = Promise.resolve();
function decorateFunction(fn, options) {
    return buildIntermediateFunction(fn, options);
}
export function memorize(...args) {
    if (typeof args[0] === 'function') {
        return decorateFunction(args[0], args[1]);
    }
    const [options] = args;
    return (...args) => buildIntermediateFunction(args[0], options);
}
export default memorize;
function buildIntermediateFunction(originalFn, { ttl = Infinity } = {}) {
    const cacheMap = new MultikeyMap();
    const name = originalFn.name;
    const nameDescriptor = Object.getOwnPropertyDescriptor(fn, 'name');
    if (nameDescriptor.configurable) {
        Object.defineProperty(fn, 'name', { value: name });
    }
    else if (nameDescriptor.writable) {
        fn.name = name;
    }
    return fn;
    function fn(...args) {
        const keys = [this, ...args];
        let [hasCache, cache] = cacheMap.hasAndGet(keys);
        if (!hasCache) {
            cache = originalFn.apply(this, args);
            cacheMap.set(keys, cache);
            if (ttl === 'async') {
                Promise.resolve(cache).then(cleanUp, cleanUp);
            }
            else if (ttl !== Infinity) {
                if (ttl === false) {
                    RESOLVED.then(cleanUp);
                }
                else {
                    const timer = setTimeout(cleanUp, ttl);
                    if (typeof timer === 'object' && typeof timer.unref === 'function') {
                        timer.unref();
                    }
                }
            }
        }
        return cache;
        function cleanUp() {
            cacheMap.delete(keys);
        }
    }
}
//# sourceMappingURL=index.js.map